declare global {
    interface Window {
        webkitRequestAnimationFrame: (callback: FrameRequestCallback) => number;
        mozRequestAnimationFrame: (callback: FrameRequestCallback) => number;
        msRequestAnimationFrame: (callback: FrameRequestCallback) => number;
        oRequestAnimationFrame: (callback: FrameRequestCallback) => number;
        webkitCancelAnimationFrame: (handle: number) => void;
        mozCancelAnimationFrame: (handle: number) => void;
        oCancelAnimationFrame: (handle: number) => void;
        msCancelAnimationFrame: (handle: number) => void;
        webkitIndexedDB: IDBFactory;
        mozIndexedDB: IDBFactory;
        msIndexedDB: IDBFactory;
        indexedDB: IDBFactory;
        SpeechRecognition: any;
        webkitSpeechRecognition: any;
    }
    interface Navigator {
        webkitGetUserMedia: () => void;
        mozGetUserMedia: () => void;
        msGetUserMedia: () => void;
        getUserMedia: () => void;
    }
}
declare type Redirect = 'follow' | 'error' | 'manual';
declare type Cache = 'default' | 'no-store' | 'reload' | 'no-cache' | 'force-cache';
declare type Mode = 'cors' | 'no-cors' | 'same-origin' | 'navigate';
declare type ResponseType = 'formData' | 'text' | 'blob' | 'arrayBuffer' | 'json';
declare type BodyType = 'form' | 'json' | 'file';
declare type Method = 'GET' | 'POST' | 'PUT' | 'DELETE' | 'OPTIONS';
declare type Credentials = 'omit' | 'include' | 'same-origin';
interface IFetchInterceptors {
    request?: {
        success?: (config: IFetchConfig) => IFetchConfig;
        error?: (error: any) => Promise<never>;
    };
    response?: {
        success?: (response: any) => any;
        error?: (error: any) => Promise<never>;
    };
    success?: (response: Response) => Response;
    error?: (error: any) => Promise<never>;
}
interface IFetchConfig extends IFetchOptions {
    url?: string;
    keepalive?: boolean;
    body?: any;
    retry?: number;
    integrity?: string;
    referrer?: string;
    referrerPolicy?: 'no-referrer' | 'no-referrer-when-downgrade' | 'origin' | 'origin-when-cross-origin' | 'unsafe-url' | 'strict-origin' | 'strict-origin-when-cross-origin' | 'same-origin';
    method?: Method;
    credentials?: Credentials;
    params?: Record<string, string>;
    responseType?: ResponseType;
    bodyType?: BodyType;
    cache?: Cache;
    redirect?: Redirect;
    mode?: Mode;
    signal?: AbortSignal;
    cancel?: () => void;
    transformResponse?: (response: Response) => Response;
}
interface IFetchOptions {
    baseURL?: string;
    timeout?: number;
    headers?: Record<string, any>;
    interceptors?: IFetchInterceptors;
}
interface DeviceType {
    os: string;
    dev: string;
}
declare type TrimType = 'all' | 'pre' | 'around' | 'post';
interface EventBus {
    data: Record<string, Function[]>;
    emit: (event: string, data?: any) => void;
    on: (event: string, fn: (data?: any) => void) => void;
    off: (event: string, fn: Function) => void;
}
interface Position {
    x: number;
    y: number;
}
declare type FileType = 'file' | 'blob' | 'url';
interface JSCookie {
    get: (key: string) => string | undefined;
    set: (key: string, value: string, extdays?: number) => void;
    delete: (key: string) => void;
}
interface LRU {
    set(key: string, value: any): void;
    get(key: string): any;
    cache: Map<string, any>;
    max: number;
    size: () => number;
}
interface ISignature {
    canvas: HTMLCanvasElement;
    ctx: CanvasRenderingContext2D;
    createCanvas(w: number, h: number): void;
    clearCanvas(): void;
}
interface FileMD5 {
    HASH: string;
    suffix: string;
    filename: string;
    buffer: ArrayBuffer;
}
interface FileChunk {
    file: Blob;
    filename: string;
}
interface Deadline {
    timeRemaining: () => number;
    didTimeout: boolean;
}
interface DragEvent {
    dragStart?: (e: any) => void;
    dragMove?: (e: any) => void;
    dragEnd?: (e: any) => void;
}
interface MutationObserverInit {
    childList?: boolean;
    attributes?: boolean;
    characterData?: boolean;
    subtree?: boolean;
    attributeOldValue?: boolean;
    characterDataOldValue?: boolean;
    attributeFilter?: string[];
}
interface ParsedURL {
    protocol?: string;
    host?: string;
    auth?: string;
    pathname: string;
    hash: string;
    search: string;
}
interface IShellMessage {
    status: number;
    result: string;
}
declare type PkgTool = 'npm' | 'yarn' | 'pnpm' | 'bun';
declare type Keys<A, B> = keyof A | keyof B;
declare type Val<key, A, B> = key extends keyof A ? key extends keyof B ? A[key] extends number | string | boolean | symbol | null | undefined ? A[key] | B[key] : Merge<A[key], B[key]> : A[key] : key extends keyof B ? B[key] : never;
declare type Merge<A, B> = {
    [key in Keys<A, B>]: Val<key, A, B>;
};
interface IsESModule {
    default: any;
}
interface NodeWorkerPayload {
    params: string | string[];
    stdio?: 'inherit' | 'pipe';
    errorExit?: boolean;
    isLog?: boolean;
}
declare type UseTimeoutReturn<T> = T extends Function ? () => void : undefined;
declare type MaybeElement = string | HTMLElement;
declare type Lang = 'zh-CN' | 'en-US' | 'ja-JP' | 'ko-KR' | 'zh-TW' | 'zh-HK' | 'zh-MO' | 'zh-SG';

export { BodyType, Cache, Credentials, Deadline, DeviceType, DragEvent, EventBus, FileChunk, FileMD5, FileType, IFetchConfig, IFetchInterceptors, IFetchOptions, IShellMessage, ISignature, IsESModule, JSCookie, LRU, Lang, MaybeElement, Merge, Method, Mode, MutationObserverInit, NodeWorkerPayload, ParsedURL, PkgTool, Position, Redirect, ResponseType, TrimType, UseTimeoutReturn };
